@@ -1,8 +1,7 @@
 package HTTP::Engine;
 use Moose;
 use HTTP::Engine::Types::Core qw( Interface );
-our $VERSION = '0.0.12';
-use HTTP::Engine::Context;
+our $VERSION = '0.0.13';
 use HTTP::Engine::Request;
 use HTTP::Engine::Request::Upload;
 use HTTP::Engine::Response;
@@ -15,44 +14,8 @@ has 'interface' => (
     handles => [ qw(run load_plugins) ],
 );
 
-sub import {
-    my($class, %args) = @_;
-    return unless $args{middlewares} && ref $args{middlewares} eq 'ARRAY';
-    $class->load_middlewares(@{ $args{middlewares} });
-}
-
-sub load_middlewares {
-    my ($class, @middlewares) = @_;
-    for my $middleware (@middlewares) {
-        $class->load_middleware( $middleware );
-    }
-}
-
-sub load_middleware {
-    my ($class, $middleware) = @_;
-
-    my $pkg;
-    if (($pkg = $middleware) =~ s/^(\+)//) {
-        Class::MOP::load_class($pkg) or die $@;
-    } else {
-        $pkg = 'HTTP::Engine::Middleware::' . $middleware;
-        unless (eval { Class::MOP::load_class($pkg) }) {
-            $pkg = 'HTTPEx::Middleware::' . $middleware;
-            Class::MOP::load_class($pkg);
-        }
-    }
-
-    if ($pkg->meta->has_method('setup')) {
-        $pkg->setup();
-    }
-
-    if ($pkg->meta->has_method('wrap')) {
-        HTTP::Engine::RequestProcessor->meta->add_around_method_modifier(
-            call_handler => $pkg->meta->get_method('wrap')->body
-        );
-    }
-}
-
+no Moose;
+__PACKAGE__->meta->make_immutable;
 1;
 __END__
 
@@ -81,8 +44,8 @@ HTTP::Engine - Web Server Gateway Interface and HTTP Server Engine Drivers (Yet
 
   use Data::Dumper;
   sub handle_request {
-      my $c = shift;
-      $c->res->body( Dumper($c->req) );
+      my $req = shift;
+      HTTP::Engine::Response->new( body => Dumper($req) );
   }
 
 
@@ -91,6 +54,14 @@ HTTP::Engine - Web Server Gateway Interface and HTTP Server Engine Drivers (Yet
 Version 0.0.x is a concept release, the internal interface is still fluid. 
 It is mostly based on the code of Catalyst::Engine.
 
+=head1 COMPATIBILITY
+
+version over 0.0.13 is incompatible of version under 0.0.12.
+
+using L<HTTP::Engine::Compat> module if you want compatibility of version under 0.0.12.
+
+version 0.0.13 is unsupported of context and middleware.
+
 =head1 DESCRIPTION
 
 HTTP::Engine is a bare-bones, extensible HTTP engine. It is not a 
@@ -109,6 +80,19 @@ HTTP::Engine will handle absorbing the differences between the environment,
 the I/O, etc. Your application can focus on creating response objects
 (which is pretty much what your typical webapp is doing)
 
+
+The community can be found via:
+
+  IRC: irc.perl.org#http-engine irc.freenode.net#coderepos
+
+  Wiki Page: http://coderepos.org/share/wiki/HTTP%3A%3AEngine
+
+  SVN: http://svn.coderepos.org/share/lang/perl/HTTP-Engine  
+
+  Trac: http://coderepos.org/share/browser/lang/perl/HTTP-Engine
+
+  Mailing list: http://lists.scsys.co.uk/cgi-bin/mailman/listinfo/http-engine
+
 =head1 INTERFACES
 
 Interfaces are the actual environment-dependent components which handles
@@ -164,23 +148,27 @@ Or you can let HTTP::Engine instantiate the interface for you:
     }
   )->run();
 
-=head1 MIDDLEWARES
-
-For all non-core middlewares (consult #codrepos first), use the HTTPEx::
-namespace. For example, if you have a plugin module named "HTTPEx::Middleware::Foo",
-you could load it as
-
-  use HTTP::Engine middlewares => [ qw( +HTTPEx::Plugin::Foo ) ];
-
-=head1 METHODS
+=head1 CONCEPT
 
 =over 4
 
-=item load_middleware(middleware)
+=item HTTP::Engine is Not
 
-=item load_middlewares(qw/ middleware middleware /)
+    session manager
+    authentication manager
+    URL dispatcher
+    model manager
+    toy
+    black magick
 
-Loads the given middleware into the HTTP::Engine.
+=item HTTP::Engine is
+
+    HTTP abstraction layer
+
+=item HTTP::Engine's ancestry
+
+    WSGI
+    Rack
 
 =back
 
@@ -210,8 +198,8 @@ kan
 
 =head1 SEE ALSO
 
-wiki page L<http://coderepos.org/share/wiki/HTTP%3A%3AEngine>
-
+L<HTTP::Engine::Compat>,
+L<HTTPEx::Declare>,
 L<Moose>
 
 =head1 REPOSITORY
