@@ -7,6 +7,8 @@ use MooseX::Types::Moose qw( Object HashRef ArrayRef CodeRef );
 
 use Class::MOP;
 use URI;
+use URI::WithBase;
+use URI::QueryParam;
 use HTTP::Headers;
 
 role_type Interface, { role => "HTTP::Engine::Role::Interface" };
@@ -26,9 +28,19 @@ coerce Interface, from HashRef => via {
     return $module->new( %$args );
 };
 
-class_type Uri, { class => "URI" };
+class_type Uri, { class => "URI::WithBase" };
 
-coerce Uri, from Str => via { URI->new($_) };
+coerce Uri, from Str => via { 
+    # generate base uri
+    my $uri = URI->new($_);
+    my $base = $uri->path;
+    $base =~ s{^/+}{};
+    $uri->path($base);
+    $base .= '/' unless $base =~ /\/$/;
+    $uri->query(undef);
+    $uri->path($base);
+    URI::WithBase->new($_, $uri);
+};
 
 class_type Header, { class => "HTTP::Headers" };
 
